part of twilio_programmable_video;

/// A remote data track publication represents a [RemoteDataTrack] that has been shared to a [Room].
class RemoteDataTrackPublication implements DataTrackPublication {
  final String _sid;
  final String _name;
  RemoteDataTrack _remoteDataTrack;
  bool _subscribed;
  bool _enabled;

  /// The SID of the published data track.
  @override
  String get trackSid => _sid;

  /// The name of the published data track.
  @override
  String get trackName => _name;

  /// Returns `true` if the published data track is enabled or `false` otherwise.
  @override
  bool get isTrackEnabled => _enabled;

  /// Returns `true` if the published data track is subscribed by the local participant or `false` otherwise.
  bool get isTrackSubscribed => _subscribed;

  /// Returns the published remote data track.
  ///
  /// Will return `null` if the track is not subscribed to.
  RemoteDataTrack get remoteDataTrack => _remoteDataTrack;

  /// The base data track object of the published remote data track.
  ///
  /// Will return `null` if the track is not subscribed to.
  @override
  DataTrack get dataTrack => _remoteDataTrack;

  RemoteDataTrackPublication(this._subscribed, this._enabled, this._sid, this._name)
      : assert(_sid != null),
        assert(_name != null);

  /// Construct from a map.
  factory RemoteDataTrackPublication._fromMap(Map<String, dynamic> map) {
    var remoteDataTrackPublication = RemoteDataTrackPublication(map['subscribed'], map['enabled'], map['sid'], map['name']);
    remoteDataTrackPublication._updateFromMap(map);
    return remoteDataTrackPublication;
  }

  /// Update properties from a map.
  void _updateFromMap(Map<String, dynamic> map) {
    _subscribed = map['subscribed'];
    _enabled = map['enabled'];

    if (map['remoteDataTrack'] != null) {
      final remoteDataTrackMap = Map<String, dynamic>.from(map['remoteDataTrack']);
      _remoteDataTrack ??= RemoteDataTrack._fromMap(remoteDataTrackMap);
      _remoteDataTrack._updateFromMap(remoteDataTrackMap);
    } else {
      _remoteDataTrack = null;
    }
  }
}
